@@ -6,12 +6,12 @@ A module implementing a base connection-arbiter interface.
 import asyncio as _asyncio
 from contextlib import AsyncExitStack as _AsyncExitStack
 from inspect import isawaitable as _isawaitable
+from logging import getLogger as _getLogger
 from typing import Awaitable as _Awaitable
 from typing import Callable as _Callable
 from typing import Iterable as _Iterable
 from typing import List as _List
 from typing import MutableMapping as _MutableMapping
-from typing import NamedTuple
 from typing import Union as _Union
 
 # third-party
@@ -23,22 +23,16 @@ from vcorelib.namespace import Namespace as _Namespace
 from vcorelib.namespace import NamespaceMixin as _NamespaceMixin
 
 # internal
+from runtimepy.net.arbiter.housekeeping import metrics_poller
+from runtimepy.net.arbiter.info import AppInfo, ConnectionMap
+from runtimepy.net.arbiter.task import (
+    ArbiterTaskManager as _ArbiterTaskManager,
+)
 from runtimepy.net.connection import Connection as _Connection
 from runtimepy.net.manager import ConnectionManager as _ConnectionManager
 
-ConnectionMap = _MutableMapping[str, _Connection]
-
-
-class AppInfo(NamedTuple):
-    """References provided to network applications."""
-
-    stack: _AsyncExitStack
-    connections: ConnectionMap
-    stop: _asyncio.Event
-    config: _JsonObject
-
-
 NetworkApplication = _Callable[[AppInfo], _Awaitable[int]]
+NetworkApplicationlike = _Union[NetworkApplication, _List[NetworkApplication]]
 ServerTask = _Awaitable[None]
 
 
@@ -49,6 +43,20 @@ async def init_only(app: AppInfo) -> int:
     return 0
 
 
+def normalize_app(
+    app: NetworkApplicationlike = None,
+) -> _List[NetworkApplication]:
+    """
+    Normalize some application parameter into a list of network applications.
+    """
+
+    if app is None:
+        app = [init_only]
+    elif not isinstance(app, list):
+        app = [app]
+    return app
+
+
 class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
     """
     A class implementing a base connection-manager for a broader application.
@@ -60,7 +68,7 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
         stop_sig: _asyncio.Event = None,
         namespace: _Namespace = None,
         logger: _LoggerType = None,
-        app: NetworkApplication = init_only,
+        app: NetworkApplicationlike = None,
         config: _JsonObject = None,
     ) -> None:
         """Initialize this connection arbiter."""
@@ -71,6 +79,11 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
             manager = _ConnectionManager()
         self.manager = manager
 
+        self.task_manager = _ArbiterTaskManager()
+
+        # Ensure that connection metrics are polled.
+        self.task_manager.register(metrics_poller(self.manager))
+
         if stop_sig is None:
             stop_sig = _asyncio.Event()
         self.stop_sig = stop_sig
@@ -79,7 +92,7 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
 
         # A fallback application. Set a class attribute so this can be more
         # easily externally updated.
-        self._app = app
+        self._apps: _List[NetworkApplication] = normalize_app(app)
 
         # Application configuration data.
         if config is None:
@@ -136,7 +149,7 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
 
     async def _entry(
         self,
-        app: NetworkApplication = None,
+        app: NetworkApplicationlike = None,
         check_connections: bool = True,
         config: _JsonObject = None,
     ) -> int:
@@ -174,18 +187,43 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
                 async with _AsyncExitStack() as stack:
                     self.logger.info("Application starting.")
 
-                    if app is None:
-                        app = self._app
-
-                    result = await app(
-                        AppInfo(
-                            stack,
-                            self._connections,
-                            self.stop_sig,
-                            config if config is not None else self._config,
-                        )
+                    info = AppInfo(
+                        self.logger,
+                        stack,
+                        self._connections,
+                        self._namespace,
+                        self.stop_sig,
+                        config if config is not None else self._config,
                     )
-                    self.logger.info("Application returned %d.", result)
+
+                    # Initialize tasks.
+                    await _asyncio.gather(
+                        *(x.init(info) for x in self.task_manager.tasks)
+                    )
+
+                    # Start tasks.
+                    await stack.enter_async_context(
+                        self.task_manager.running(stop_sig=self.stop_sig)
+                    )
+
+                    # Get application methods.
+                    apps = self._apps
+                    if app is not None:
+                        apps = normalize_app(app)
+
+                    result = 0
+                    for curr_app in apps:
+                        if result == 0:
+                            info.logger = _getLogger(curr_app.__name__)
+                            info.logger.info("Starting.")
+                            try:
+                                result = await curr_app(info)
+                                info.logger.info("Returned %d.", result)
+                            except AssertionError as exc:
+                                info.logger.exception(
+                                    "Failed an assertion:", exc_info=exc
+                                )
+                                result = -1
 
         finally:
             for conn in self._connections.values():
@@ -196,7 +234,7 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
 
     async def app(
         self,
-        app: NetworkApplication = None,
+        app: NetworkApplicationlike = None,
         check_connections: bool = True,
         config: _JsonObject = None,
     ) -> int:
@@ -206,7 +244,7 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
 
         result = await _asyncio.gather(
             self._entry(
-                app, check_connections=check_connections, config=config
+                app=app, check_connections=check_connections, config=config
             ),
             self.manager.manage(self.stop_sig),
             *self._servers,
@@ -215,7 +253,7 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
 
     def run(
         self,
-        app: NetworkApplication = None,
+        app: NetworkApplicationlike = None,
         eloop: _asyncio.AbstractEventLoop = None,
         signals: _Iterable[int] = None,
         check_connections: bool = True,
