@@ -6,10 +6,19 @@ A module aggregating commonly used connection factory classes.
 from runtimepy.net.arbiter.tcp import TcpConnectionFactory
 from runtimepy.net.arbiter.udp import UdpConnectionFactory
 from runtimepy.net.arbiter.websocket import WebsocketConnectionFactory
-from runtimepy.net.tcp.connection import EchoTcpConnection, TcpConnection
-from runtimepy.net.udp.connection import EchoUdpConnection, UdpConnection
-from runtimepy.net.websocket.connection import (
+from runtimepy.net.tcp import (
+    EchoTcpConnection,
+    NullTcpConnection,
+    TcpConnection,
+)
+from runtimepy.net.udp import (
+    EchoUdpConnection,
+    NullUdpConnection,
+    UdpConnection,
+)
+from runtimepy.net.websocket import (
     EchoWebsocketConnection,
+    NullWebsocketConnection,
     WebsocketConnection,
 )
 
@@ -18,12 +27,15 @@ __all__ = [
     "TcpConnection",
     "TcpConnectionFactory",
     "TcpEcho",
+    "TcpNull",
     "UdpConnection",
     "UdpConnectionFactory",
     "UdpEcho",
+    "UdpNull",
     "WebsocketConnection",
     "WebsocketConnectionFactory",
     "WebsocketEcho",
+    "WebsocketNull",
 ]
 
 
@@ -33,13 +45,31 @@ class UdpEcho(UdpConnectionFactory[EchoUdpConnection]):
     kind = EchoUdpConnection
 
 
+class UdpNull(UdpConnectionFactory[NullUdpConnection]):
+    """UDP null-connection factory."""
+
+    kind = NullUdpConnection
+
+
 class TcpEcho(TcpConnectionFactory[EchoTcpConnection]):
     """TCP echo-connection factory."""
 
     kind = EchoTcpConnection
 
 
+class TcpNull(TcpConnectionFactory[NullTcpConnection]):
+    """TCP null-connection factory."""
+
+    kind = NullTcpConnection
+
+
 class WebsocketEcho(WebsocketConnectionFactory[EchoWebsocketConnection]):
     """WebSocket echo-connection factory."""
 
     kind = EchoWebsocketConnection
+
+
+class WebsocketNull(WebsocketConnectionFactory[NullWebsocketConnection]):
+    """WebSocket null-connection factory."""
+
+    kind = NullWebsocketConnection
