@@ -29,6 +29,7 @@ from runtimepy.net import sockname as _sockname
 from runtimepy.net.connection import BinaryMessage as _BinaryMessage
 from runtimepy.net.connection import Connection as _Connection
 from runtimepy.net.connection import EchoConnection as _EchoConnection
+from runtimepy.net.connection import NullConnection as _NullConnection
 from runtimepy.net.manager import ConnectionManager as _ConnectionManager
 from runtimepy.net.mixin import (
     BinaryMessageQueueMixin as _BinaryMessageQueueMixin,
@@ -73,6 +74,10 @@ ConnectionCallback = _Callable[[T], None]
 class TcpConnection(_Connection, _TransportMixin):
     """A TCP connection interface."""
 
+    # TCP connections send data directly without going through queues.
+    uses_text_tx_queue = False
+    uses_binary_tx_queue = False
+
     def __init__(self, transport: _Transport, protocol: QueueProtocol) -> None:
         """Initialize this TCP connection."""
 
@@ -85,21 +90,22 @@ class TcpConnection(_Connection, _TransportMixin):
         self._protocol.conn = self
         super().__init__(_getLogger(self.logger_name("TCP ")))
 
-        # TCP connections send data directly without going through queues.
-        self.uses_text_tx_queue = False
-        self.uses_binary_tx_queue = False
-
     async def _await_message(self) -> _Optional[_Union[_BinaryMessage, str]]:
         """Await the next message. Return None on error or failure."""
-        return await self._protocol.queue.get()
+
+        data = await self._protocol.queue.get()
+        if data is not None:
+            self.metrics.rx.increment(len(data))
+        return data
 
     def send_text(self, data: str) -> None:
         """Enqueue a text message to send."""
-        self._transport.write(data.encode())
+        self.send_binary(data.encode())
 
     def send_binary(self, data: _BinaryMessage) -> None:
         """Enqueue a binary message tos end."""
         self._transport.write(data)
+        self.metrics.tx.increment(len(data))
 
     @classmethod
     async def create_connection(cls: _Type[T], **kwargs) -> T:
@@ -203,3 +209,7 @@ class TcpConnection(_Connection, _TransportMixin):
 
 class EchoTcpConnection(TcpConnection, _EchoConnection):
     """An echo connection for TCP."""
+
+
+class NullTcpConnection(TcpConnection, _NullConnection):
+    """A null TCP connection."""
